/* { dg-do run } */
/* { dg-require-effective-target cilkplus_runtime } */
/* { dg-options "-fcilkplus -save-temps" } */
/* { dg-additional-options "-std=gnu99" { target c } } */

#include "pr69826-1.c"
